getInvoiceTopLevelItems = [
    {
        'categoryCode': 'sov_sec_ip_addresses_priv',
        'createDate': '2018-04-04T23:15:20-06:00',
        'description': '64 Portable Private IP Addresses',
        'id': 724951323,
        'oneTimeAfterTaxAmount': '0',
        'recurringAfterTaxAmount': '0',
        'hostName': 'bleg',
        'domainName': 'beh.com',
        'category': {'name': 'Private (only) Secondary VLAN IP Addresses'},
        'children': [
            {
                'id': 12345,
                'category': {'name': 'Fake Child Category'},
                'description': 'Blah',
                'oneTimeAfterTaxAmount': 55.50,
                'recurringAfterTaxAmount': 0.10
            }
        ],
        'location': {'name': 'fra02'}
    }
]
